package com.bsb.incubator.vaadin.addressbook;

import com.bsb.incubator.vaadin.addressbook.container.InMemoryPersonContainer;
import com.bsb.incubator.vaadin.addressbook.container.PersonContainer;
import com.bsb.incubator.vaadin.addressbook.ui.HelpWindow;
import com.bsb.incubator.vaadin.addressbook.ui.ListView;
import com.bsb.incubator.vaadin.addressbook.ui.NavigationTree;
import com.bsb.incubator.vaadin.addressbook.ui.PersonForm;
import com.bsb.incubator.vaadin.addressbook.ui.PersonList;
import com.bsb.incubator.vaadin.addressbook.ui.SearchView;
import com.bsb.incubator.vaadin.addressbook.ui.SharingOptionsWindow;
import com.bsb.incubator.vaadin.addressbook.ui.internal.SearchFilter;
import com.vaadin.Application;
import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.event.ItemClickEvent;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.Component;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.SplitPanel;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;

public class AddressBookApplication extends Application
        implements Button.ClickListener, Property.ValueChangeListener,
        ItemClickEvent.ItemClickListener {


    private final Button newContact = new Button("Add contact");
    private final Button search = new Button("Search");
    private final Button share = new Button("Share");
    private final Button help = new Button("Help");
    private final SplitPanel horizontalSplit = new SplitPanel(
            SplitPanel.ORIENTATION_HORIZONTAL);
    private final NavigationTree navigationTree = new NavigationTree(this);

    private final InMemoryPersonContainer dataSource =
            InMemoryPersonContainer.createWithTestData();
    private final PersonForm personForm = new PersonForm(this);


    // Lazy loaded fields
    private ListView listView;
    private PersonList personList;
    private SearchView searchView;
    private HelpWindow helpWindow;
    private SharingOptionsWindow sharingOptionsWindow;


    @Override
    public void init() {
        buildMainLayout();
        setTheme("contracts");
    }


    public void buildMainLayout() {
        setMainWindow(new Window("Address Book"));


        final VerticalLayout layout = new VerticalLayout();
        layout.setSizeFull();

        layout.addComponent(createToolbar());
        layout.addComponent(horizontalSplit);
        /* Allocate all available extra space to the horizontal split panel */
        layout.setExpandRatio(horizontalSplit, 1);
        /* Set the initial split position so we can have a 200 pixel menu to the left */
        horizontalSplit.setSplitPosition(200, SplitPanel.UNITS_PIXELS);

        horizontalSplit.setFirstComponent(navigationTree);
        showListView();

        getMainWindow().setContent(layout);
    }

    public HorizontalLayout createToolbar() {
        HorizontalLayout lo = new HorizontalLayout();
        lo.addComponent(newContact);
        lo.addComponent(search);
        lo.addComponent(share);
        lo.addComponent(help);

        lo.setMargin(true);
        lo.setSpacing(true);

        newContact.setIcon(new ThemeResource("icons/32/document-add.png"));
        search.setIcon(new ThemeResource("icons/32/folder-add.png"));
        share.setIcon(new ThemeResource("icons/32/users.png"));
        help.setIcon(new ThemeResource("icons/32/help.png"));


        final Embedded em = new Embedded("", new ThemeResource("images/logo.png"));
        lo.addComponent(em);
        lo.setComponentAlignment(em, Alignment.MIDDLE_RIGHT);

        lo.setWidth("100%");
        lo.setExpandRatio(em, 1);
        lo.setStyleName("toolbar");

        newContact.addListener((Button.ClickListener) this);
        search.addListener((Button.ClickListener) this);
        help.addListener((Button.ClickListener) this);
        share.addListener((Button.ClickListener) this);

        return lo;
    }


    @Override
    public void buttonClick(Button.ClickEvent event) {
        final Button source = event.getButton();
        if (source == search) {
            showSearchView();
        }
        if (source == help) {
            getMainWindow().addWindow(getHelpWindow());
        }
        if (source == share) {
            getMainWindow().addWindow(getSharingOptionsWindow());

        }
        if (source == newContact) {
            addNewContact();
        }
    }

    @Override
    public void valueChange(Property.ValueChangeEvent event) {
        Property property = event.getProperty();
        if (property == getPersonList()) {
            Item item = getPersonList().getItem(getPersonList().getValue());
            if (item != personForm.getItemDataSource()) {
                personForm.setItemDataSource(item);
            }
        }
    }

    @Override
    public void itemClick(ItemClickEvent event) {
        if (event.getSource() == navigationTree) {
            Object itemId = event.getItemId();
            if (itemId != null) {
                if (NavigationTree.SHOW_ALL.equals(itemId)) {
                    // clear previous filters
                    getDataSource().removeAllContainerFilters();
                    showListView();
                } else if (NavigationTree.SEARCH.equals(itemId)) {
                    showSearchView();
                } else if (itemId instanceof SearchFilter) {
                    // If the user clicks on a save search in the tree
                    search((SearchFilter) itemId);
                }
            }
        }
    }

    private void showListView() {
        setMainComponent(getListView());
    }

    private void showSearchView() {
        // Switch the main component to the search view
        setMainComponent(getSearchView());
    }

    private void addNewContact() {
        showListView();
        personForm.addContact();
    }

    public void search(SearchFilter searchFilter) {
        // clear previous filters
        getDataSource().removeAllContainerFilters();
        // filter contacts with given filter
        getDataSource().addContainerFilter(searchFilter.getPropertyId(),
                searchFilter.getTerm(), true, false);

        getMainWindow().showNotification(
                "Searched for " + searchFilter.getPropertyId() + "=*"
                        + searchFilter.getTerm() + "*, found "
                        + getDataSource().size() + " item(s).",
                Window.Notification.TYPE_TRAY_NOTIFICATION);

        showListView();
    }

    public void saveSearch(SearchFilter searchFilter) {
        navigationTree.addItem(searchFilter);
        navigationTree.setParent(searchFilter, NavigationTree.SEARCH);
        // mark the saved search as a leaf (cannot have children)
        navigationTree.setChildrenAllowed(searchFilter, false);
        // make sure "Search" is expanded
        navigationTree.expandItem(NavigationTree.SEARCH);
        // select the saved search
        navigationTree.setValue(searchFilter);
    }


    // Fields lazy loading


    public PersonContainer getDataSource() {
        return dataSource;
    }

    private void setMainComponent(Component c) {
        horizontalSplit.setSecondComponent(c);
    }

    private ListView getListView() {
        if (listView == null) {
            listView = new ListView(getPersonList(), personForm);
        }
        return listView;
    }

    public PersonList getPersonList() {
        if (personList == null) {
            personList = new PersonList(this);
        }
        return personList;
    }

    public SearchView getSearchView() {
        if (searchView == null) {
            searchView = new SearchView(this);
        }
        return searchView;
    }

    public HelpWindow getHelpWindow() {
        if (helpWindow == null) {
            helpWindow = new HelpWindow();
        }
        return helpWindow;
    }

    public SharingOptionsWindow getSharingOptionsWindow() {
        if (sharingOptionsWindow == null) {
            sharingOptionsWindow = new SharingOptionsWindow();
        }
        return sharingOptionsWindow;
    }
}
