package com.bsb.incubator.vaadin.addressbook.ui;

import com.bsb.incubator.vaadin.addressbook.AddressBookApplication;
import com.bsb.incubator.vaadin.addressbook.container.PersonContainer;
import com.bsb.incubator.vaadin.addressbook.domain.Person;
import com.vaadin.data.Item;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.validator.EmailValidator;
import com.vaadin.data.validator.RegexpValidator;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.DefaultFieldFactory;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.TextField;

import java.util.Arrays;
import java.util.List;

/**
 * The form displayed in the right panel to view, edit and create person
 * instances.
 *
 * @author Stephane Nicoll
 */
public class PersonForm extends Form implements Button.ClickListener {

    private final Button save = new Button("Save", (Button.ClickListener) this);
    private final Button cancel = new Button("Cancel", (Button.ClickListener) this);
    private final Button edit = new Button("Edit", (Button.ClickListener) this);
    private final ComboBox cities = new ComboBox("City");
    private final AddressBookApplication app;

    private boolean newContactMode = false;
    private Person newPerson = null;

    public PersonForm(AddressBookApplication app) {
        this.app = app;

        /* Allow the user to enter new cities */
        cities.setNewItemsAllowed(true);
        /* We do not want to use null values */
        cities.setNullSelectionAllowed(false);
        /* Add an empty city used for selecting no city */
        cities.addItem("");
        /* Populate cities select using the cities in the data container */
        PersonContainer ds = app.getDataSource();
        for (Object o : ds.getItemIds()) {
            final Person person = (Person) o;
            cities.addItem(person.getCity());
        }

        // Needed to add validators and customize the behavior
        setFormFieldFactory(new DefaultFieldFactory() {
            @Override
            public Field createField(Item item, Object propertyId,
                                     Component uiContext) {
                if (propertyId.equals("city")) {
                    return cities;
                }
                Field field = super.createField(item, propertyId, uiContext);
                if (field instanceof TextField) {
                    // We do not want to display "null" to the user when the
                    // field is empty.
                    ((TextField) field).setNullRepresentation("");
                }


                // Validation
                if (propertyId.equals("postalCode")) {
                    final TextField tf = (TextField) field;

                    /* Add a validator for postalCode and make it required */
                    tf.addValidator(new RegexpValidator("[1-9][0-9]{4}",
                            "Postal code must be a five digit number and cannot start with a zero."));
                    tf.setRequired(true);
                }
                if (propertyId.equals("email")) {
                    final TextField tf = (TextField) field;

                    /* Add a validator for postalCode and make it required */
                    tf.addValidator(new EmailValidator("Invalid email address."));
                }

                return field;
            }
        });


        // Enable buffering so that commit() must be called for the form
        // before input is written to the data. (Form input is not written
        // immediately through to the underlying object.)
        setWriteThrough(false);

        final HorizontalLayout footer = new HorizontalLayout();
        footer.setSpacing(true);
        footer.addComponent(save);
        footer.addComponent(cancel);
        footer.addComponent(edit);
        footer.setVisible(false);

        setFooter(footer);
    }

    @Override
    public void setItemDataSource(Item newDataSource) {
        newContactMode = false;
        if (newDataSource != null) {
            List<Object> orderedProperties = Arrays
                    .asList(PersonContainer.NATURAL_COL_ORDER);
            super.setItemDataSource(newDataSource, orderedProperties);
            setReadOnly(true);
            getFooter().setVisible(true);
        } else {
            super.setItemDataSource(null);
            getFooter().setVisible(false);
        }
    }

    @Override
    public void setReadOnly(boolean readOnly) {
        super.setReadOnly(readOnly);
        save.setVisible(!readOnly);
        cancel.setVisible(!readOnly);
        edit.setVisible(readOnly);
    }

    public void buttonClick(Button.ClickEvent event) {
        Button source = event.getButton();
        if (source == save) {
            // If the given input is not valid there is no point in continuing
            if (!isValid()) {
                return;
            }
            commit();
            if (newContactMode) {
                /* We need to add the new person to the container */
                final Item addedItem = app.getDataSource().addItem(newPerson);
                /*
                * We must update the form to use the Item from our datasource
                * as we are now in edit mode
                */
                setItemDataSource(addedItem);
                newContactMode = false;
            }

            setReadOnly(true);
        } else if (source == cancel) {
            if (newContactMode) {
                newContactMode = false;
                setItemDataSource(null);
            } else {
                discard();
            }
            setReadOnly(true);
        } else if (source == edit) {
            setReadOnly(false);
        }
    }

    public void addContact() {
        // Create a temporary item for the form
        newPerson = new Person();
        setItemDataSource(new BeanItem<Person>(newPerson));
        newContactMode = true;
        setReadOnly(false);
    }
}

